@@ -98,7 +98,8 @@ setMethod('posterior', signature(model = "BFlinearModel", index = "missing", dat
       stop("Sampling from intercept-only model not implemented.")
     }else if(all(relevantDataTypes == "continuous")){
       ## Regression
-      chains = linearReg.Gibbs(y = data[[dv]],covariates = data[factors],iterations = iterations, rscale = rscaleCont, ...)
+      X = fullDesignMatrix(formula, data, dataTypes)
+      chains = linearReg.Gibbs(y = data[[dv]],covariates = X,iterations = iterations, rscale = rscaleCont, ...)
     }else if(all(relevantDataTypes != "continuous")){
       # ANOVA or t test
       chains = nWayFormula(formula=formula, data = data, 
@@ -106,7 +107,7 @@ setMethod('posterior', signature(model = "BFlinearModel", index = "missing", dat
                        rscaleFixed = rscaleFixed,
                        rscaleRandom = rscaleRandom,
                        iterations = iterations,
-                       gibbs = TRUE, ...) 
+                       posterior = TRUE, ...) 
     }else{
       # GLM
       chains = nWayFormula(formula=formula, data = data, 
@@ -115,7 +116,7 @@ setMethod('posterior', signature(model = "BFlinearModel", index = "missing", dat
                        rscaleRandom = rscaleRandom,
                        rscaleCont = rscaleCont,
                        iterations = iterations,
-                       gibbs = TRUE, ...)        
+                       posterior = TRUE, ...)        
     }
     
     return(new("BFmcmc",chains, model = model, data = data))
@@ -192,7 +193,7 @@ setMethod('posterior', signature(model = "BFproportion", index = "missing", data
 ## S3
 ###########
 
-as.mcmc.BFmcmc <- function(x){
+as.mcmc.BFmcmc <- function(x, ...){
   return(S3Part(x))
 }
 
